<?php
namespace Ep\Validator;

use GuzzleHttp\ClientInterface;
use Zend\Validator\AbstractValidator;

/**
 * Validira reCaptcha odgovor.
 *
 * @package Ep\Google\Validator
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/core
 */
class ReCaptcha extends AbstractValidator
{
	const MISSING_INPUT_SECRET = 'missing-input-secret';
	const INVALID_INPUT_SECRET = 'invalid-input-secret';
	const MISSING_INPUT_RESPONSE = 'missing-input-response';
	const INVALID_INPUT_RESPONSE = 'invalid-input-response';
	const BAD_REQUEST = 'bad-request';

	/** @var array */
	protected $messageTemplates = [
		self::MISSING_INPUT_SECRET => 'The secret parameter is missing.',
		self::INVALID_INPUT_SECRET => 'The secret parameter is invalid or malformed.',
		self::MISSING_INPUT_RESPONSE => 'The response parameter is missing.',
		self::INVALID_INPUT_RESPONSE => 'The response parameter is invalid or malformed.',
		self::BAD_REQUEST => 'The request is invalid or malformed.',
	];

	/** @var ClientInterface */
	protected $http;
	/** @var string */
	protected $secretKey;

    /**
     * ReCaptcha constructor.
     *
     * @param ClientInterface $http
     * @param null $options
     */
	public function __construct(ClientInterface $http, $options = null)
	{
        parent::__construct($options);

        $this->http      = $http;
        $this->secretKey = $options['secret_key'] ?? '';

        if (function_exists('_x')) {
            $this->messageTemplates = [
                self::MISSING_INPUT_SECRET => _x('Tajni parametar nedostaje.', 'recaptcha validator', 'ep'),
                self::INVALID_INPUT_SECRET => _x('Tajni parametar je neispravan ili izobličen.', 'recaptcha validator', 'ep'),
                self::MISSING_INPUT_RESPONSE => _x('Parametar odgovora nedostaje.', 'recaptcha validator', 'ep'),
                self::INVALID_INPUT_RESPONSE => _x('Parametar odgovora je neispravan ili izobličen.', 'recaptcha validator', 'ep'),
                self::BAD_REQUEST => _x('Upit je neispravan ili izobličen.', 'recaptcha validator', 'ep'),
            ];
        }
	}

	/**
	 * @return string
	 */
	public function getSecretKey(): string
	{
		return $this->secretKey;
	}

	/**
	 * @param string $secretKey
	 */
	public function setSecretKey(string $secretKey)
	{
		$this->secretKey = $secretKey;
	}

    /**
     * Returns true if and only if $value meets the validation requirements
     *
     * If $value fails validation, then this method returns false, and
     * getMessages() will return an array of messages that explain why the
     * validation failed.
     *
     * @param  mixed $value
     *
     * @return bool
     *
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
	public function isValid($value)
	{
		if (empty($value)) {
			$this->error(self::MISSING_INPUT_RESPONSE);
			return false;
		}

		if (empty($this->secretKey)) {
			$this->error(self::MISSING_INPUT_SECRET);
			return false;
		}

        $response = $this->http->request('POST', 'siteverify', [
			'form_params' => [
				'secret' => $this->secretKey,
				'response' => $value,
			],
		]);

		$json = (string) $response->getBody();
		$data = json_decode($json, true);

		if (null === $data) {
			throw new \RuntimeException(sprintf(
				"Došlo je do pogreške prilikom dekodiranja JSON-a: ",
				json_last_error_msg()
			));
		}

		if (true !== $data['success']) {
			foreach ($data['error-codes'] as $errorCode) {
				$this->error($errorCode);
			}
			return false;
		}

		return true;
	}
}